@@ -5,14 +5,8 @@ import os
 import os.path
 import pickle
 import pprint
-import sys
 import warnings
-from typing import Any, Dict, Iterator, List, Optional, Sequence, Tuple, Union, overload
-
-if sys.version_info >= (3, 8):
-    from typing import Literal
-else:
-    from typing_extensions import Literal
+from typing import Any, Iterator, Literal, Sequence, overload
 
 from . import core
 from .exceptions import RTreeError
@@ -240,7 +234,6 @@ class Index:
             # assume if the file exists, we're not going to overwrite it
             # unless the user explicitly set the property to do so
             if os.path.exists(p):
-
                 self.properties.overwrite = bool(kwargs.get("overwrite", False))
 
                 # assume we're fetching the first index_id.  If the user
@@ -253,7 +246,7 @@ class Index:
 
             d = os.path.dirname(p)
             if not os.access(d, os.W_OK):
-                message = "Unable to open file '%s' for index storage" % f
+                message = f"Unable to open file '{f}' for index storage"
                 raise OSError(message)
         elif storage:
             self.properties.storage = RT_Custom
@@ -303,16 +296,14 @@ class Index:
             return 0
 
     def __repr__(self) -> str:
-        return "rtree.index.Index(bounds={}, size={})".format(
-            self.bounds, self.get_size()
-        )
+        return f"rtree.index.Index(bounds={self.bounds}, size={self.get_size()})"
 
-    def __getstate__(self) -> Dict[str, Any]:
+    def __getstate__(self) -> dict[str, Any]:
         state = self.__dict__.copy()
         del state["handle"]
         return state
 
-    def __setstate__(self, state: Dict[str, Any]) -> None:
+    def __setstate__(self, state: dict[str, Any]) -> None:
         self.__dict__.update(state)
         self.handle = IndexHandle(self.properties.handle)
 
@@ -338,8 +329,7 @@ class Index:
 
     def get_coordinate_pointers(
         self, coordinates: Sequence[float]
-    ) -> Tuple[float, float]:
-
+    ) -> tuple[float, float]:
         try:
             iter(coordinates)
         except TypeError:
@@ -583,22 +573,21 @@ class Index:
         return p_num_results.value
 
     @overload
-    def contains(self, coordinates: Any, objects: Literal[True]) -> Iterator[Item]:
-        ...
+    def contains(self, coordinates: Any, objects: Literal[True]) -> Iterator[Item]: ...
 
     @overload
     def contains(
         self, coordinates: Any, objects: Literal[False] = False
-    ) -> Optional[Iterator[int]]:
-        ...
+    ) -> Iterator[int] | None: ...
 
     @overload
-    def contains(self, coordinates: Any, objects: Literal["raw"]) -> Iterator[object]:
-        ...
+    def contains(
+        self, coordinates: Any, objects: Literal["raw"]
+    ) -> Iterator[object]: ...
 
     def contains(
-        self, coordinates: Any, objects: Union[bool, Literal["raw"]] = False
-    ) -> Optional[Iterator[Union[Item, int, object]]]:
+        self, coordinates: Any, objects: bool | Literal["raw"] = False
+    ) -> Iterator[Item | int | object] | None:
         """Return ids or objects in the index that contains within the given
         coordinates.
 
@@ -732,24 +721,23 @@ class Index:
         return new_idx
 
     @overload
-    def intersection(self, coordinates: Any, objects: Literal[True]) -> Iterator[Item]:
-        ...
+    def intersection(
+        self, coordinates: Any, objects: Literal[True]
+    ) -> Iterator[Item]: ...
 
     @overload
     def intersection(
         self, coordinates: Any, objects: Literal[False] = False
-    ) -> Iterator[int]:
-        ...
+    ) -> Iterator[int]: ...
 
     @overload
     def intersection(
         self, coordinates: Any, objects: Literal["raw"]
-    ) -> Iterator[object]:
-        ...
+    ) -> Iterator[object]: ...
 
     def intersection(
-        self, coordinates: Any, objects: Union[bool, Literal["raw"]] = False
-    ) -> Iterator[Union[Item, int, object]]:
+        self, coordinates: Any, objects: bool | Literal["raw"] = False
+    ) -> Iterator[Item | int | object]:
         """Return ids or objects in the index that intersect the given
         coordinates.
 
@@ -835,7 +823,6 @@ class Index:
         return self._get_ids(it, p_num_results.value)
 
     def _intersectionTP(self, coordinates, velocities, times, objects=False):
-
         p_mins, p_maxs = self.get_coordinate_pointers(coordinates)
         pv_mins, pv_maxs = self.get_coordinate_pointers(velocities)
         t_start, t_end = self._get_time_doubles(times)
@@ -868,7 +855,6 @@ class Index:
             return self._get_ids(it, p_num_results.value)
 
     def _intersection_obj(self, coordinates, objects):
-
         p_mins, p_maxs = self.get_coordinate_pointers(coordinates)
 
         p_num_results = ctypes.c_uint64(0)
@@ -886,7 +872,6 @@ class Index:
         return self._get_objects(it, p_num_results.value, objects)
 
     def _contains_obj(self, coordinates: Any, objects):
-
         p_mins, p_maxs = self.get_coordinate_pointers(coordinates)
 
         p_num_results = ctypes.c_uint64(0)
@@ -927,10 +912,8 @@ class Index:
                     else:
                         yield self.loads(data)
 
+        finally:
             core.rt.Index_DestroyObjResults(its, num_results)
-        except Exception:  # need to catch all exceptions, not just rtree.
-            core.rt.Index_DestroyObjResults(its, num_results)
-            raise
 
     def _get_ids(self, it, num_results):
         # take the pointer, yield the results  and free
@@ -940,13 +923,11 @@ class Index:
         try:
             for i in range(num_results):
                 yield items.contents[i]
+
+        finally:
             core.rt.Index_Free(its)
-        except Exception:
-            core.rt.Index_Free(its)
-            raise
 
     def _nearest_obj(self, coordinates, num_results, objects):
-
         p_mins, p_maxs = self.get_coordinate_pointers(coordinates)
 
         p_num_results = ctypes.pointer(ctypes.c_uint64(num_results))
@@ -967,27 +948,24 @@ class Index:
     @overload
     def nearest(
         self, coordinates: Any, num_results: int, objects: Literal[True]
-    ) -> Iterator[Item]:
-        ...
+    ) -> Iterator[Item]: ...
 
     @overload
     def nearest(
         self, coordinates: Any, num_results: int, objects: Literal[False] = False
-    ) -> Iterator[int]:
-        ...
+    ) -> Iterator[int]: ...
 
     @overload
     def nearest(
         self, coordinates: Any, num_results: int, objects: Literal["raw"]
-    ) -> Iterator[object]:
-        ...
+    ) -> Iterator[object]: ...
 
     def nearest(
         self,
         coordinates: Any,
         num_results: int = 1,
-        objects: Union[bool, Literal["raw"]] = False,
-    ) -> Iterator[Union[Item, int, object]]:
+        objects: bool | Literal["raw"] = False,
+    ) -> Iterator[Item | int | object]:
         """Returns the ``k``-nearest objects to the given coordinates.
 
         :param coordinates: This may be an object that satisfies the numpy array
@@ -1180,7 +1158,7 @@ class Index:
         return core.rt.Index_ClearBuffer(self.handle)
 
     @classmethod
-    def deinterleave(self, interleaved: Sequence[object]) -> List[object]:
+    def deinterleave(self, interleaved: Sequence[object]) -> list[object]:
         """
         [xmin, ymin, xmax, ymax] => [xmin, xmax, ymin, ymax]
 
@@ -1199,7 +1177,7 @@ class Index:
         return di
 
     @classmethod
-    def interleave(self, deinterleaved: Sequence[float]) -> List[float]:
+    def interleave(self, deinterleaved: Sequence[float]) -> list[float]:
         """
         [xmin, xmax, ymin, ymax, zmin, zmax]
             => [xmin, ymin, zmin, xmax, ymax, zmax]
@@ -1388,7 +1366,7 @@ class Item:
         return self.id > other.id
 
     @property
-    def bbox(self) -> List[float]:
+    def bbox(self) -> list[float]:
         """Returns the bounding box of the index entry"""
         return Index.interleave(self.bounds)
 
@@ -1418,7 +1396,6 @@ class Handle:
 
     def destroy(self) -> None:
         try:
-
             if self._ptr is not None:
                 self._destroy(self._ptr)
                 self._ptr = None
@@ -1443,7 +1420,6 @@ class Handle:
 
 
 class IndexHandle(Handle):
-
     _create = core.rt.Index_Create
     _destroy = core.rt.Index_Destroy
 
@@ -1457,12 +1433,10 @@ class IndexHandle(Handle):
 
 
 class IndexStreamHandle(IndexHandle):
-
     _create = core.rt.Index_CreateWithStream
 
 
 class PropertyHandle(Handle):
-
     _create = core.rt.IndexProperty_Create
     _destroy = core.rt.IndexProperty_Destroy
 
@@ -1506,19 +1480,19 @@ class Property:
         self.handle = handle
         self.initialize_from_dict(kwargs)
 
-    def initialize_from_dict(self, state: Dict[str, Any]) -> None:
+    def initialize_from_dict(self, state: dict[str, Any]) -> None:
         for k, v in state.items():
             if v is not None:
                 setattr(self, k, v)
 
-    def __getstate__(self) -> Dict[Any, Any]:
+    def __getstate__(self) -> dict[Any, Any]:
         return self.as_dict()
 
     def __setstate__(self, state):
         self.handle = PropertyHandle()
         self.initialize_from_dict(state)
 
-    def as_dict(self) -> Dict[str, Any]:
+    def as_dict(self) -> dict[str, Any]:
         d = {}
         for k in self.pkeys:
             try:
@@ -2123,8 +2097,8 @@ class RtreeContainer(Rtree):
                 or isinstance(args[0], bytes)
                 or isinstance(args[0], ICustomStorage)
             ):
-                raise ValueError("%s supports only in-memory indexes" % self.__class__)
-        self._objects: Dict[int, Tuple[int, object]] = {}
+                raise ValueError(f"{self.__class__} supports only in-memory indexes")
+        self._objects: dict[int, tuple[int, object]] = {}
         return super().__init__(*args, **kwargs)
 
     def get_size(self) -> int:
@@ -2190,18 +2164,16 @@ class RtreeContainer(Rtree):
     add = insert  # type: ignore[assignment]
 
     @overload  # type: ignore[override]
-    def intersection(self, coordinates: Any, bbox: Literal[True]) -> Iterator[Item]:
-        ...
+    def intersection(self, coordinates: Any, bbox: Literal[True]) -> Iterator[Item]: ...
 
     @overload
     def intersection(
         self, coordinates: Any, bbox: Literal[False] = False
-    ) -> Iterator[object]:
-        ...
+    ) -> Iterator[object]: ...
 
     def intersection(
         self, coordinates: Any, bbox: bool = False
-    ) -> Iterator[Union[Item, object]]:
+    ) -> Iterator[Item | object]:
         """Return ids or objects in the index that intersect the given
         coordinates.
 
@@ -2271,18 +2243,16 @@ class RtreeContainer(Rtree):
     @overload  # type: ignore[override]
     def nearest(
         self, coordinates: Any, num_results: int = 1, bbox: Literal[True] = True
-    ) -> Iterator[Item]:
-        ...
+    ) -> Iterator[Item]: ...
 
     @overload
     def nearest(
         self, coordinates: Any, num_results: int = 1, bbox: Literal[False] = False
-    ) -> Iterator[object]:
-        ...
+    ) -> Iterator[object]: ...
 
     def nearest(
         self, coordinates: Any, num_results: int = 1, bbox: bool = False
-    ) -> Iterator[Union[Item, object]]:
+    ) -> Iterator[Item | object]:
         """Returns the ``k``-nearest objects to the given coordinates
         in increasing distance order.
 
@@ -2386,5 +2356,5 @@ class RtreeContainer(Rtree):
                 [self._objects[child_id][1] for child_id in child_ids],
                 bounds,
             )
-            for id, child_ids, bounds in super(RtreeContainer, self).leaves()
+            for id, child_ids, bounds in super().leaves()
         ]
