@@ -42,12 +42,6 @@ class IndexTestCase(unittest.TestCase):
         assert next(rtree_test.intersection([1, 1, 1, 2, 2, 2])) == 1564
 
 
-class IndexVersion(unittest.TestCase):
-    def test_libsidx_version(self) -> None:
-        self.assertTrue(index.major_version == 1)
-        self.assertTrue(index.minor_version >= 7)
-
-
 class IndexCount(unittest.TestCase):
     def setUp(self) -> None:
         self.boxes15 = np.genfromtxt("boxes_15x15.data")
@@ -262,7 +256,7 @@ class IndexIntersection(IndexTestCase):
         hit = [h for h in hits if h.id == 4321][0]
         self.assertEqual(hit.id, 4321)
         self.assertEqual(hit.object, 42)
-        box = ["%.10f" % t for t in hit.bbox]
+        box = [f"{t:.10f}" for t in hit.bbox]
         expected = ["34.3776829412", "26.7375853734", "49.3776829412", "41.7375853734"]
         self.assertEqual(box, expected)
 
